/**
* This sample shows how to use an analog input sequence acquisition using DMA_SINGLE mode.
**/
#include "stdafx.h"

#define NB_CHANNELS 16

/**
* Pointer that will be allocated and that will enables to read the data of the board
**/
USHORT * data = NULL;
DWORD	dataSize = 0;
FILE *ptrFile;
bool createTxtLog = false;

/**
* Interrupt routine that enables to get the data read by the board
**/
static void SequenceInterruptRoutine (HANDLE	deviceHandle,
							   BYTE		fifoOverflow,
							   DWORD	interruptSource,
							   DWORD	interruptMask,
							   PWORD	args,
							   DWORD	argsCount )
{
	LONG read = 0;
	DWORD i = 0;

	UNREFERENCED_PARAMETER(args);
	/* switch through the possible interrupt sources */
	/* check the interrupt fifo overflow */
	if (fifoOverflow || interruptSource & 0x10 || interruptSource & 1) {
		if (fifoOverflow) 
			fprintf(stderr,"Error: interrupt fifo overflow. That means that your software do not handle the interrupts fast enough.\n");
		if (interruptSource & 0x10)
			fprintf(stderr,"Error: PLD FIFO overflow. The board is too slow to handle all the data\n");
	}

	/* DMA */
	if (interruptSource & 0x4) {
		if (interruptMask & 0x4) {
			printf("internal sequence fifo overflow \n");
			return;
		} else {			
			// Get the sequence data 
			read = l_PCIe3126_ReadSequenceData( deviceHandle, dataSize, data);
			if (read < 0) {
				fprintf(stderr, "Error %d in l_PCIe3126_ReadSequenceData \n", (int) read);
				return;
			} 
			else {		

				DWORD i = 0;
				/* display the values */
				for( i = 0 ; i < argsCount ; i++ )
				{
					if ( i % 4 == 0 )
					{
						if (createTxtLog)
							fprintf(ptrFile, "\n");
						else
							printf("\n");
					}
					if (createTxtLog)
						fprintf(ptrFile, "0x%.4x ", data[i]);
					else
						printf("0x%.4x ", data[i] );
				}
				if (createTxtLog)
					fprintf(ptrFile, "\n");
				else
					printf("\n");

			}
		}
	}

	/* End of DMA acquisition */
	if (interruptSource & 0x8)
		fprintf(stdout, "End of DMA acquisition (You get the number of sequence asked).\n");
}

int _tmain()
{
	INT returnValue = 0;
	HANDLE boardHandle = NULL;
	ULONG readValue = 0;
	BYTE channels[16] = {0};
	BYTE gain[16] = {0};
	BYTE polarity[16] = {0};
	BYTE inputMode = 0;
	ULONG convertingTime = 0;
	ULONG sequenceCount = 0;
	DWORD sequenceBeforeInterrupt = 0;
		
	/* initialize the 4 channels of the acquisition */
	channels[0] = 0;
	channels[1] = 1;
	channels[2] = 2;
	channels[3] = 3;

	gain[0] = PCIE3126_GAIN_1;
	gain[1] = PCIE3126_GAIN_1;
	gain[2] = PCIE3126_GAIN_1;
	gain[3] = PCIE3126_GAIN_1;

	polarity[0] = PCIE3126_OUTPUT_MODE_BIPOLAR;
	polarity[1] = PCIE3126_OUTPUT_MODE_BIPOLAR;
	polarity[2] = PCIE3126_OUTPUT_MODE_BIPOLAR;
	polarity[3] = PCIE3126_OUTPUT_MODE_BIPOLAR;

	/* open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d opening the board \n", returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* This sample will initialize an acquisition using 4 channels and no delay. \n");
	printf("* The sequence is : \n");
	printf("*   - Channel 0, bipolar, gain 1 \n");
	printf("*   - Channel 1, bipolar, gain 1 \n");
	printf("*   - Channel 2, bipolar, gain 1 \n");
	printf("*   - Channel 3, bipolar, gain 1 \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* ask the input mode */
	ReadUlong("Select the input mode (0: Single-ended, 1: Differential) ", 0, 1, &readValue);
	switch (readValue)
	{
	case 0:
		inputMode = PCIE3126_INPUT_MODE_SINGLE_ENDED;
		break;
	case 1:
		inputMode = PCIE3126_INPUT_MODE_DIFFERENTIAL;
		break;
	}

	/* set the input mode */
	returnValue = i_PCIe3126_SetAnalogInputMode( boardHandle, inputMode );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3126_SetAnalogInputMode\n", returnValue);
		i_PCIe3126_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* set the board interrupt routine */
	returnValue = i_PCIe3126_SetBoardIntRoutine( boardHandle, SequenceInterruptRoutine );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3126_SetBoardIntRoutine \n", returnValue);
		i_PCIe3126_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* Ask for log file */
	ReadUlong("Do you want to log the data in txt file.\nHighly recommend if the acquistion is fast (0: No, 1: Yes) ", 0, 1, &readValue);

	switch (readValue)
	{
	case 0:
		createTxtLog = 0;
		break;
	case 1:
		createTxtLog = 1;
		ptrFile = fopen("log.txt", "w+");
		if (ptrFile == NULL)
		{
			printf("Error with the log file...\n");
			return -1;
		}
		break;
	}

	/* ask the number of sequences to acquire */
	ReadUlong("How many sequences do you want to acquire (0: continuous) ? ", 0, 0xFFFFFFFF, &sequenceCount );

	/* ask the number of sequences before interrupt */
	ReadUlong("How many sequences do you want before the board generate an interrupt ? ", 1, 0xFFFFFFFF, &sequenceBeforeInterrupt );

	/* allocate memory */
	dataSize = sequenceBeforeInterrupt * 4 * sizeof(USHORT);
	data = (USHORT*)malloc( dataSize );
	if ( data == NULL )
	{
		printf("Error in malloc. data is null \n" );
		i_PCIe3126_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* ask the converting time */
	ReadUlong("What converting time do you want to use? (5 to 65535000 - us) ", 5, 65535000, &convertingTime );
	
	printf("While the acquisition is running, press any key to stop the sample \n");
	printf("Press a key to start the acquisition \n");
	_getch();
	printf("Acquistion start... \n");

	/* start the analog input sequence acquisition */
	returnValue = i_PCIe3126_InitAndStartAnalogInputSequence(boardHandle,
		sequenceCount,					/* number of sequences to acquire (0 for continuous)*/
		sequenceBeforeInterrupt,		/* number of sequences in one interrupt */
		4,								/* 4 channels */
		channels,
		gain,
		polarity,
		convertingTime,
		PCIE3126_NO_DELAY,				/* no delay */
		PCIE3126_SOFT_TRIGGER_DISABLED,	/* no software trigger */
		PCIE3126_EXT_TRIGGER_DISABLED,	/* no hardware trigger */
		0);

	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3126_StartAnalogInputSequence \n", returnValue);
		i_PCIe3126_ResetBoardIntRoutine(boardHandle);
		i_PCIe3126_CloseBoard(boardHandle);
		_getch();
		return -1;
	}

	/* 
	* endless loop, waiting an input from the user 
	* See the InterruptRoutine in the beginning of this file to see how the messages are displayed
	*/
	for(;;)
	{
		/* use hit keyboard ?*/
		if (_kbhit())
		{
			break;
		}
		Sleep(100);
	}
	
	/* stop the acquisition */
	returnValue = i_PCIe3126_StopAnalogAcquisition( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3126_StopAnalogAcquisition \n", returnValue);
	}
	else
	{
		printf("i_PCIe3126_StopAnalogAcquisition OK \n");
	}

	/* reset the interrupt routine */
	returnValue = i_PCIe3126_ResetBoardIntRoutine( boardHandle );
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3126_ResetBoardIntRoutine \n", returnValue);
	}
	else
	{
		printf("i_PCIe3126_ResetBoardIntRoutine OK \n");
	}

	/* close the board */
	returnValue = i_PCIe3126_CloseBoard(boardHandle);
	if (returnValue != 0)
	{
		printf("Error %d in i_PCIe3126_CloseBoard \n", returnValue);
	}
	else
	{
		printf("i_PCIe3126_CloseBoard OK \n");
	}

	/* free memory */
	free(data);
	if (createTxtLog)
		fclose(ptrFile);
	/* exit */
	getch();
	getch();
	return 0;
}

