// Here goes prototypes declaration and documentation

#include <windows.h>
#include "define.h" /* this include file stores typedef that can be easily used */


#ifdef __cplusplus 
	extern "C" {
#endif


/** \defgroup PCIe3126 xPCIe-3126 functions
 * @{
 *		\addtogroup General xPCIe-3126 general functions
 *		\ingroup PCIe3126
 *		@{ 
 */
 
/**  Enables to get the number of xPCIe-3126 boards present in the system. \n
 * The function fill the value pointed by pb_NbrOfBoards with the number of boards present.
 *                                                                            
 * @param[out] pb_NbrOfBoards Value that will be filled with the number of boards     
 * @return	0 : No error \n
 *		   -1 : Error while getting the number of boards \n
 */
_INT_ i_PCIe3126_GetNumberOfBoards (PBYTE	pb_NbrOfBoards );

/** Get device information of the board.
 *                                                                   
 * @param[in] b_BoardIndex : Index of the board
 * @param[in] dw_IdentifierStringSize : size of the identifier parameter 
 * @param[out] pc_Identifier: Identifier of the board
 * @param[out] pdw_UINumber : This number is typically a user-perceived slot number, such as a number printed next to the slot on the board, or some other number that makes locating the physical device easier for the user. If the device is on a bus that has no UI number convention, or if the bus driver for the device cannot determine the UI number, this value is 0xFFFFFFFF.
 * @param[out] pdw_DeviceNumber : Device number.
 * @param[out] pdw_BusNumber : Bus number.
 * @param[out] pdw_BaseAddress0 : Base address 0 of the board
 * @param[out] pdw_BaseAddress1 : Base address 1 of the board
 * @param[out] pdw_BaseAddress2 : Base address 2 of the board
 * @param[out] pdw_BaseAddress3 : Base address 3 of the board
 * @param[out] pb_Interrupt : Interrupt line of the board
 * @return	 0 : No error \n
 *			-1 : Unable to open the board via index \n
 *			-2 : Error when calling driver's IOCTL \n
 */
_INT_ i_PCIe3126_GetBoardInformation(BYTE		b_BoardIndex,
									DWORD		dw_IdentifierStringSize,
									PCHAR		pc_Identifier,
									PDWORD		pdw_UINumber,
									PDWORD		pdw_DeviceNumber,
									PDWORD		pdw_BusNumber,
									PDWORD		pdw_BaseAddress0,
									PDWORD		pdw_BaseAddress1,
									PDWORD		pdw_BaseAddress2,
									PDWORD		pdw_BaseAddress3,
									PBYTE		pb_Interrupt );

/** Get revision of the board.
 *
 * @param[in] b_BoardIndex : Index of the board
 * @param[out] pdw_BoardRevision: Revision number of the board
 * @return	 0 : No error \n
 *			-1 : Unable to open the board via index \n
 *			-2 : Error when calling driver's IOCTL \n
 */
_INT_ i_PCIe3126_GetBoardRevision(BYTE		b_BoardIndex,
									PDWORD		pdw_BoardRevision);

/** Open the board with the index : b_BoardIndex. A handle is returned to the \n    
 * user which allows to use the driver functions. \n 
 *                                                                                  
 * @param[in] b_BoardIndex : Index of the board to open                             
 * @param[out] ph_DeviceHandle : Handle of the selected xPCIe-3126 board for using the functions
 * @return		0 : No error
 *             -1 : Unable to get the number of available boards \n                                    
 *             -2 : Wrong device index parameter. The parameter is too high \n
 *             -3 : Error while getting the device's class \n
 *             -4 : Error while getting informations about the device \n
 *             -5 : Memory allocation error \n
 *             -6 : Get device interface detail error \n
 *             -7 : Error while creating communication handle \n
 *			   -8 : Error in the initialization callback function \n
 */
_INT_ i_PCIe3126_OpenBoardViaIndex (BYTE	b_BoardIndex,
								   PHANDLE	ph_DeviceHandle );    

/** Open the board with the indentifier : pc_Identifier. A handle is returned to the \n    
 * user which allows to use the following functions. \n                                                         
 *                                                                                  
 * @param[in] pc_Identifier : Identifier of the board to open
 * @param[out] ph_DeviceHandle : Handle of the board xPCIe-3126 for using the functions
 * @return	 0 : No Error \n
			-1 : Unable to get the number of available boards \n  
			-2 : Unable to get the board identifier \n
			-3 : Unable to open the board \n
			-4 : Identifier not found \n
 */
_INT_ i_PCIe3126_OpenBoardViaIdentifier (PCHAR	pc_Identifier,
										PHANDLE	ph_DeviceHandle );
										
/** Release the handle of the board. Block the access to the board.
 *                                                                   
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @return	 0 : No Error \n
 *			-1 : The device handle parameter is wrong \n
 */
_INT_ i_PCIe3126_CloseBoard (HANDLE	h_DeviceHandle);

/** Return the version of the DLL.
 *                                                                   
 * @param[out] pdw_Version  : Version. sample : 0x01010910 -> version 1.01 september 2010       
 * @return	0 : No Error \n
 */
_INT_ i_PCIe3126_GetDLLVersion (PDWORD	pdw_Version);

/*Function's Name   : i_PCIe3126_GetDeviceDriverVersion*/
/** Return the version of the device driver.
*                                                                   
* @param[in] h_DeviceHandle  : Handle of the board                
* @param[out] pdw_Version  : Version. sample : 0x01011211 -> version 1.01 - 12/2011
* @return    0 : No Error \n                                                     
*             -1 : Error while calling IOCTL function \n
*/
_INT_ i_PCIe3126_GetDeviceDriverVersion(HANDLE h_DeviceHandle, PDWORD pdw_Version);

/**
 *		@}
 *		
 *		\addtogroup LoadLogic xPCIe3126 Update the FPGA firmware via the PCI bus
 * 		\ingroup PCIe3126
 *		@{
 */

/* i_PCIe3126_LoadLogicCommand */
/**
 * 
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   dw_Command LoadLogic command
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe3126_LoadLogicCommand(IN  HANDLE h_DeviceHandle,
                                 IN  DWORD dw_Command);


/* i_PCIe3126_LoadLogicWrite */
/**
 * 
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[in]   dw_Data LoadLogic write data
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe3126_LoadLogicWrite(IN  HANDLE h_DeviceHandle,
                               IN  DWORD dw_Data);


/* i_PCIe3126_LoadLogicRead */
/**
 * 
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  dw_Data LoadLogic read data
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe3126_LoadLogicRead(IN  HANDLE h_DeviceHandle,
                              OUT DWORD *dw_Data);


/* i_PCIe3126_GetLoadLogicStatus */
/**
 * 
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  b_Status LoadLogic status
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe3126_GetLoadLogicStatus(IN  HANDLE h_DeviceHandle,
                                   OUT BYTE *b_Status);


/* i_PCIe3126_GetFirmwareVersion */
/**
 * 
 *
 * @param[in]   h_DeviceHandle Handle of the board
 * @param[out]  dw_Version Current firmware version
 * @retval  0   No error
 * @retval -1   The h_DeviceHandle parameter is wrong
 * @retval -100 An error occurred while calling the IOCTL function
 */
_INT_ i_PCIe3126_GetFirmwareVersion(IN  HANDLE h_DeviceHandle,
                                   OUT DWORD *dw_Version);

/**		@}
 *		\addtogroup Interrupt xPCIe-3126 interrupt management functions
 * 		\ingroup PCIe3126
 *		@{ 
 */
 
/** Install an interrupt callback function.
 * This function will be called each time the board generate an interruption. \n
 * The value of dw_InterruptSource will enable you to determine the source of the interruption:
 * <ul>
 *	<li>0x1: End of sequence interrupt. When you use i_PCIe3126_Read1AnalogInput and i_PCIe3126_ReadMoreAnalogInputs with interrupt, you will receive this interrupt. The values are then available thanks to pw_Args parameter</li>
 *	<li>0x2: Timer interrupt. </li>
 *	<li>0x4: DMA interrupt. You will receive this interrupt when using i_PCIe3126_InitAndStartAnalogInputSequence. When you get this interrupt, you have to read the data measured by the board by calling the function l_PCIe3126_ReadSequenceData (see its own documentation). If your computer is too slow to handle all the data, after a while you will have an internal data fifo overflow (dw_InterruptMask will be 0x4). You can change the size of the internal data fifo in the driver's configuration pages.</li>
 *	<li>0x8: End of DMA acquisition. You will receive this interrupt when using i_PCIe3126_InitAndStartAnalogInputSequence in non continuous mode</li>
 *	<li>0x10: PLD FIFO overflow. The board is too slow to handle all the data</li>
 * </ul> 
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @param[in] v_FunctionName    : Address of the callback function
 *		\n	\b h_DeviceHandle	: Handle of the board
 *		\n	\b b_FIFOOverflow	: If uc_FIFOOverflow != 0, there are too many untreated interrupts in the kernel FIFO, and any new information is lost
 *		\n  \b dw_InterruptSource : Source of the interrupt
 *		\n	\b dw_InterruptMask	: Source of the interrupt
 *		\n	\b pw_Args			: Values read during the interrupt. If DMA interrupt, this pointer will be set to NULL.
 *		\n	\b dw_ArgsCount		: Number of values read during the interrupt
 *
 * @return	 0 : No Error
 *			-1 : The device handle parameter is wrong \n
 *			-2 : An interrupt routine was still isntalled. Remove it with i_PCIe3126_ResetBoardIntRoutine and retry \n
 *			-3 : Error while creating kill event \n
 *			-4 : Error while creating interrupt event \n
 *			-5 : Error while creating interrupt thread \n
 *			-6 : Error while changing the priority class of the thread \n
 *			-7 : Error while changing the priority of the thread \n
 *			-8 : Error while calling the IOCTL command \n
 *			-9 : Error in the interrupt callback function \n
 */
_INT_ i_PCIe3126_SetBoardIntRoutine (HANDLE	h_DeviceHandle,
									void	(*v_FunctionName)(	HANDLE	h_DeviceHandle,
																BYTE	b_FIFOOverflow,
																DWORD	dw_InterruptSource,
																DWORD	dw_InterruptMask,
																PWORD	pw_Args,
																DWORD	dw_ArgsCount )
);

/** Deinstall the interrupt callback function.
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @return	 0 : No Error
 *			-1 : The device handle parameter is wrong \n
 *			-2 : No interrupt routine installed \n
 *			-3 : Error in the reset interrupt callback function \n
 */
_INT_ i_PCIe3126_ResetBoardIntRoutine (HANDLE	h_DeviceHandle );

/**
 *		@}
 *		
 *		\addtogroup Digital xPCIe-3126 digital inputs/outputs functions
 * 		\ingroup PCIe3126
 *		@{ 
 */

/** Read 1 digital input of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to read (0 to 3)
 * @param[out] pb_Value Status of the digital input
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Invalid channel \n           
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Read1DigitalInput(HANDLE	h_DeviceHandle,
								  BYTE		b_Channel,
								  PBYTE		pb_Value );

/** Read the 4 digital inputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_Value Status of the digital inputs
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Read4DigitalInputs(HANDLE	h_DeviceHandle,
								   PBYTE	pb_Value );

/** Set 1 digital output of the board to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to set (0 to 3)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n           
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Set1DigitalOutputOn(HANDLE	h_DeviceHandle,
								  BYTE		b_Channel );

/** Set 1 digital output of the board to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to reset (0 to 3)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n          
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Set1DigitalOutputOff(HANDLE	h_DeviceHandle,
									 BYTE	b_Channel );

/** Set the digital outputs set in the mask to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to set (0x0 to 0xF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong mask \n         
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Set4DigitalOutputsOn(HANDLE	h_DeviceHandle,
									 BYTE	b_Mask );

/** Set the digital outputs set in the mask to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to reset (0x0 to 0xF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong mask \n             
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Set4DigitalOutputsOff(HANDLE	h_DeviceHandle,
									  BYTE	b_Mask );

/** Enable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until calling i_PCIe3126_Set1DigitalOutputOff or i_PCIe3126_Set4DigitalOutputsOff. \n
 * When digital output memory is disabled, the active channels are reset at each call of i_PCIe3126_Set1DigitalOutputOn or i_PCIe3126_Set4DigitalOutputsOn.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_SetDigitalOutputMemoryOn(HANDLE h_DeviceHandle );

/** Disable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until calling i_PCIe3126_Set1DigitalOutputOff or i_PCIe3126_Set4DigitalOutputsOff. \n
 * When digital output memory is disabled, the active channels are reset at each call of i_PCIe3126_Set1DigitalOutputOn or i_PCIe3126_Set4DigitalOutputsOn.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : 
 */
_INT_ i_PCIe3126_SetDigitalOutputMemoryOff(HANDLE h_DeviceHandle );

/** Get the status of 1 digital output of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel Channel to get (0 to 3)
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Get1DigitalOutputStatus(HANDLE	h_DeviceHandle,
										BYTE	b_Channel,
										PBYTE	pb_OutputsStatus );

/** Get the status of all the digital outputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Get4DigitalOutputsStatus(HANDLE	h_DeviceHandle,
										 PBYTE	pb_OutputsStatus );

/**
 *		@}
 *		
 *		\addtogroup TTL xPCI-3126 TTL inputs/outputs functions 
 * 		\ingroup PCIe3126
 *		@{ 
 */
 
/** Set the TTL programmable port to input or output
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] b_InOrOut Configure the programmable port as input (PCIE3126_TTL_INPUT) or output (PCIE3126_TTL_OUTPUT)
 *
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : Invalid port mode
 * @retval -3 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCIe3126_SetTTLPortConfiguration(HANDLE	h_DeviceHandle,
										BYTE	b_InOrOut );

/** Read the TTL inputs \n
 * In order to read the programmable port (with parameter PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH), it has to be set as an input first. \n
 * If PCIE3126_PORT_INPUT or PCIE3126_PORT_PROGRAMMABLE is used, the 8 lowest bits represent the port inputs. \n
 * If PCIe3126_PORT_BOTH is used, then the 8 low bits represent the input port, and the 8 high bits the programmable port.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Port Port to read (PCIE3126_PORT_INPUT, PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH)
 * @param[out] pw_Value Value of the port.
 *
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : Invalid port selection
 * @retval -3 : The programmable port is not configured as an input
 * @retval -4 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCIe3126_ReadTTLInputs(HANDLE	h_DeviceHandle,
							  BYTE		b_Port,
							  PWORD		pw_Value );

/** Set the TTL outputs to LOW \n
 * In order to set the programmable port (with parameter PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH), it has to be set as an output first. \n
 * If PCIE3126_PORT_OUTPUT or PCIE3126_PORT_PROGRAMMABLE is used, the 8 lowest bits represent the port outputs. \n
 * If PCIe3126_PORT_BOTH is used, then the 8 low bits represent the output port, and the 8 high bits the programmable port.
 *                                                
 
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Port Port to reset (PCIE3126_PORT_OUTPUT, PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH)
 * @param[in] w_Mask Outputs to reset (0x00 to 0xFF, or 0x0000 to 0xFFFF if both ports are used)
 *
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : Invalid port selection
 * @retval -3 : The programmable port is not configured as an output
 * @retval -4 : Invalid mask
 * @retval -5 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCIe3126_SetTTLOutputsOff(HANDLE	h_DeviceHandle,
								 BYTE	b_Port,
								 WORD	w_Mask );

/** Set the TTL outputs to HIGH \n
 * In order to set the programmable port (with parameter PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH), it has to be set as an output first. \n
 * If PCIE3126_PORT_OUTPUT or PCIE3126_PORT_PROGRAMMABLE is used, the 8 lowest bits represent the port outputs. \n
 * If PCIe3126_PORT_BOTH is used, then the 8 low bits represent the output port, and the 8 high bits the programmable port.
 *                                                
 
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Port Port to reset (PCIE3126_PORT_OUTPUT, PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH)
 * @param[in] w_Mask Outputs to reset (0x00 to 0xFF, or 0x0000 to 0xFFFF if both ports are used)
 *
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : Invalid port selection
 * @retval -3 : The programmable port is not configured as an output
 * @retval -4 : Invalid mask
 * @retval -5 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCIe3126_SetTTLOutputsOn(HANDLE	h_DeviceHandle,
								BYTE	b_Port,
								WORD	w_Mask );

/** Get the TTL outputs status \n
 * In order to set the programmable port (with parameter PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH), it has to be set as an output first. \n
 * If PCIE3126_PORT_OUTPUT or PCIE3126_PORT_PROGRAMMABLE is used, the 8 lowest bits represent the port outputs. \n
 * If PCIe3126_PORT_BOTH is used, then the 8 low bits represent the output port, and the 8 high bits the programmable port.
 *                                                
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Port Port to reset (PCIE3126_PORT_OUTPUT, PCIE3126_PORT_PROGRAMMABLE or PCIe3126_PORT_BOTH)
 * @param[out] pw_OutputsStatus Outputs status (0x00 to 0xFF, or 0x0000 to 0xFFFF if both ports are used)
 *
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : Invalid port selection
 * @retval -3 : The programmable port is not configured as an output
 * @retval -4 : Invalid mask
 * @retval -5 : Error while calling IOCTL function (wrong handle ?)
 */
_INT_ i_PCIe3126_GetTTLOutputsStatus(HANDLE	h_DeviceHandle,
									BYTE	b_Port,
									PWORD	pw_OutputsStatus );

/**
 *		@}
 *		
 *		\addtogroup TCW xPCI-3126 Timer/Counter/Watchdog functions 
 * 		\ingroup PCIe3126
 *		@{ 
 */

/**
* Enables to start the timer. \n
* @param[in] h_DeviceHandle Handle of the board
* @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
* @param[in] b_TimerMode Specify the mode of the timer. (PCIE3126_TIMER_MODE_2, PCIE3126_TIMER_MODE_3). \n
* @param[in] b_TimeBase Select the time base (PCIE3126_TIME_BASE_MICROSECONDS, PCIE3126_TIME_BASE_MILLISECONDS, PCIE3126_TIME_BASE_SECONDS)
* @param[in] b_OutputMode Set the way the output will change (PCIE3126_OUTPUT_ENABLED, PCIE3126_OUTPUT_DISABLED, PCIE3126_OUTPUT_INVERTED)
* @param[in] b_InterruptEnabled Enable to use or not use an interrupt (0,1).
* @param[in] w_ReloadValue The time to count (depend on b_TimeBase)
* @param[in] b_WatchdogEnabled Specify if the watchdog is used (0,1). \n
* PCIE3126_TIMER_MODE_2: It functions like a divide-by-w_ReloadValue counter. It is used to generate a real time clock interrupt. OUT is initially high after the initialization. When the initial count has decremented to 1, OUT goes low for one CLK pulse. OUT then goes high again, the counter reloads the initial count (w_ReloadValue) and the process is repeated. This action can generate an interrupt. \n
* PCIE3126_TIMER_MODE_3: This mode is similar to mode 2 except for the duty cycle of OUT. OUT will initially be high after the initialisation. When the initial count (ul_ReloadValue) has expired, OUT goes low for the next sequence. When the initial count (ul_ReloadValue) of that sequence has expired, OUT goes high again for the following sequence. The mode is periodic; the sequence above is repeated indefinitely. \n
* @return	 0 : success \n
*           -1 : b_TimerNb is invalid \n
*		    -2 : b_TimerMode is invalid \n
*			-3 : b_TimeBase is invalid \n
*		    -4 : b_OutputMode is invalid \n
*			-5 : b_InterruptEnabled is invalid \n
*           -6 : b_WatchdogEnabled is invalid \n
*		    -7 : Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCIe3126_InitAndStartTimer(HANDLE h_DeviceHandle,
                                   BYTE b_TimerNb,
                                   BYTE b_TimerMode,
                                   BYTE b_TimeBase,
                                   BYTE b_OutputMode,
                                   BYTE b_InterruptEnabled,
                                   WORD w_ReloadValue,
                                   BYTE b_WatchdogEnabled);

/** Stops the timer \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_StopTimer(HANDLE h_DeviceHandle, BYTE b_TimerNb);

/** Reads the timer status \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 * @param[out] pb_TimerStatus Timer status (0 = disabled, 1 = enabled)
 * @param[out] pb_SoftwareTriggerStatus Status of the software trigger (0 = no software trigger occured, 1 = software trigger occured)
 * @param[out] pw_TimerValue Current value of the timer
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_ReadTimerStatus(HANDLE	h_DeviceHandle,
								 BYTE   b_TimerNb,
                                 PBYTE	pb_TimerStatus,
                                 PBYTE	pb_SoftwareTriggerStatus,
                                 PWORD	pw_TimerValue);

/** Read the value of the timer \n
 * 
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 * @param[out] pw_TimerValue Current value of the timer
 *
 * @return	 0 : No error \n
 *          -1 : b_TimerNb is invalid \n
 *		    -2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_ReadTimerValue(HANDLE	h_DeviceHandle,
                                BYTE    b_TimerNb,
								PWORD	pw_TimerValue);

/** Triggers the timer \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerNb Specify the timer to be used. (0 or 1). \n
 *
 * @return	 0 : No Error \n
 *          -1 : b_TimerNb is invalid \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_TriggerTimer(HANDLE h_DeviceHandle, BYTE b_TimerNb);


/**
 *		@}
 *		
 *		\addtogroup Analog xPCIe-3126 Analog inputs/outputs functions
 * 		\ingroup PCIe3126
 *		@{ 
 */

 /** Set analog input mode to single-ended or differential input \n
 * The input mode has to be set at least once before any call to an analog input function. \n
 * Once set, the configuration will last until the program ends, or another mode is set. \n
 * \n
 * In single-ended mode, 16 analog inputs are available. \n
 * In differential mode, there are only 8 analog inputs available. \n
 * Please refer to the manual for further information. \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_InputMode Input mode (PCIE3126_INPUT_MODE_SINGLE_ENDED or PCIE3126_INPUT_MODE_DIFFERENTIAL)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : Invalid input mode \n
 */
_INT_ i_PCIe3126_SetAnalogInputMode(HANDLE	h_DeviceHandle,
								   BYTE		b_InputMode );
 
 /** Get analog input mode (single-ended or differential input) \n
 * \n
 * In single-ended mode, 16 analog inputs are available. \n
 * In differential mode, there are only 8 analog inputs available. \n
 * Please refer to the manual for further information. \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_InputMode Input mode (PCIE3126_INPUT_MODE_UNDEFINED, PCIE3126_INPUT_MODE_SINGLE_ENDED or PCIE3126_INPUT_MODE_DIFFERENTIAL)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 */
_INT_ i_PCIe3126_GetAnalogInputMode(HANDLE	h_DeviceHandle,
								   PBYTE	pb_InputMode );
/** Initialize and starts an analog acquisition \n
* To use this function you must set an interrupt routine where you will handle the data, and you must also define the input mode. \n
* Gain acts as a multiplier for the input voltage. Possible gain values are :
* <ul>
* 	<li> \b PCIE3126_GAIN_1 : Input range is between -10V and 10V (bipolar)</li>
* 	<li> \b PCIE3126_GAIN_2 : Input range is between -5V and 5V (bipolar)</li>
* 	<li> \b PCIE3126_GAIN_5 : Input range is between -2V and 2V (bipolar)</li>
* 	<li> \b PCIE3126_GAIN_10 : Input range is between -1V and 1V (bipolar)</li>
* </ul>
* The converting time stands for the time spent to acquire one channel. The time to acquire a sequence is dw_ConvertingTime * b_ChannelCount \n
* The software trigger is a combination of 2 parameters
* <ul>
* 	<li>The first one enables or disabled the trigger</li>
*	<ul>
*	 	<li> \b PCIE3126_SOFT_TRIGGER_ENABLED : The trigger is enabled</li>
*		<li> \b PCIE3126_SOFT_TRIGGER_DISABLED : The trigger is disabled</li>
*	</ul>
* 	<li>The trigger type is selected with one of the following parameters</li>
* 	<ul>
*		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
* 		<li> \b PCIE3126_TRIGGER_SEQUENCE_TRIGGER : A pre-defined number of sequence is acquired : dw_SequenceTriggerCount</li>
* 		<li> \b PCIE3126_TRIGGER_SINGLE_SEQUENCE : One whole sequence is read for each trigger</li>
*	</ul>
* </ul>
* The external trigger is a combination of the following parameters :
* <ul>
* 	<li>The first one defines the trigger edge</li>
*	<ul>
*		<li> \b PCIE3126_EXT_TRIGGER_DISABLED : Trigger is disabled</li>
*		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_LOW : Triggers on low edge</li>
*		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_HIGH : Triggers on high edge</li>
*		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_BOTH : Triggers on both edges</li>
*	</ul>
*	<li>The second one defines the trigger type</li>
	<ul>
*		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
*		<li> \b PCIE3126_TRIGGER_SEQUENCE_TRIGGER : A pre-defined number of sequence is acquired : dw_SequenceTriggerCount</li>
*		<li> \b PCIE3126_TRIGGER_SINGLE_SEQUENCE : One whole sequence is read for each trigger</li>
* 	</ul>
* </ul>
* @param[in] h_DeviceHandle Handle of the board
* @param[in] dw_SequenceCount: The total number of sequence to acquire. Set it to 0 if you want to use continuous acquisition (i.e. infinite acquisition)
* @param[in] dw_SequenceBeforeInterrupt: Specify the number of sequence to wait before generating an interruption. For high frequency, set a high number (512 if using maximum frequency for example).
* @param[in] b_ChannelCount: The number of channels in one sequence. Rule: (b_ChannelCount * dw_SequenceBeforeInterrupt) must be a multiple of 2
* @param[in] pb_Channel Define the channels in the sequence (single ended : 0 to 15, differential : 0 to 7). Depending on your board, this number may be different.
* @param[in] pb_Gain Define the gain for each channel
* @param[in] pb_Polarity Define the polarity for each channel (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
* @param[in] dw_ConvertingTime Define the time between two channels acquisition (5us to 65 535 000us). If you choose 16 channels with a conversion time of 5us, an entire sequence will be acquired in 16 * 5 = 80us
* @param[in] dw_DelayTime Define the time between two sequences acquisition (5us to 65 535 000us).
* @param[in] b_SoftTriggerMode Defines the software trigger
* @param[in] b_ExtTriggerMode Defines the external trigger
* @param[in] dw_SequenceTriggerCount If you choose a trigger with mode PCIE3126_TRIGGER_SEQUENCE_TRIGGER, it defines the number of sequence to acquire when a trigger occurs (1 to 0xFFFFFFFF). If not use, give 0.
* @param[in] dw_HardwareTriggerCount Number of pulses that fail before the next trigger occurs (0 -> 0xFFFF)
* @return		  0 : success \n
*				 -1 : The device handle parameter is wrong \n
*		    	 -2 : The sequence size is wrong \n
*				 -3 : The input mode has not been set \n
*				 -4 : A channel index is invalid (single-ended mode) \n
*				 -5 : A gain value is wrong \n
*				 -6 : A polarity value is wrong \n
*				 -7 : The converting time is invalid \n
*				 -8 : The delay time is invalid \n
*				 -9 : The interrupt routine is not installed \n
*				-10 : dw_SequenceBeforeInterrupt is not valid \n
*				-11 : (b_ChannelCount * dw_SequenceBeforeInterrupt) is not a multiple of 2 \n
*				-12 : both software and hardware trigger are enabled. In that case, they must have the same type (one shot, sequence trigger or single sequence) \n
*				-14 : dw_HardwareTriggerCount parameter is wrong, must be < 0xFFFF
*				-13 : Error while calling IOCTL function (wrong handle ?) \n
*/
_INT_ i_PCIe3126_InitAndStartAnalogInputSequenceEx(HANDLE	h_DeviceHandle,
												DWORD	dw_SequenceCount,
												DWORD	dw_SequenceBeforeInterrupt,
												BYTE	b_ChannelCount,
												PBYTE	pb_Channel,
												PBYTE	pb_Gain,
												PBYTE	pb_Polarity,
												DWORD	dw_ConvertingTime,
												DWORD	dw_DelayTime,
												BYTE	b_SoftTriggerMode,
												BYTE	b_ExtTriggerMode,
												DWORD	dw_SequenceTriggerCount,
												DWORD   dw_HardwareTriggerCount);

 /** Initialize and starts an analog acquisition \n
 * To use this function you must set an interrupt routine where you will handle the data, and you must also define the input mode. \n
 * Gain acts as a multiplier for the input voltage. Possible gain values are :
 * <ul>
 * 	<li> \b PCIE3126_GAIN_1 : Input range is between -10V and 10V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_2 : Input range is between -5V and 5V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_5 : Input range is between -2V and 2V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_10 : Input range is between -1V and 1V (bipolar)</li>
 * </ul>
 * The converting time stands for the time spent to acquire one channel. The time to acquire a sequence is dw_ConvertingTime * b_ChannelCount \n
 * The software trigger is a combination of 2 parameters
 * <ul>
 * 	<li>The first one enables or disabled the trigger</li>
 *	<ul>
 *	 	<li> \b PCIE3126_SOFT_TRIGGER_ENABLED : The trigger is enabled</li>
 *		<li> \b PCIE3126_SOFT_TRIGGER_DISABLED : The trigger is disabled</li>
 *	</ul>
 * 	<li>The trigger type is selected with one of the following parameters</li>
 * 	<ul>
 *		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
 * 		<li> \b PCIE3126_TRIGGER_SEQUENCE_TRIGGER : A pre-defined number of sequence is acquired : dw_SequenceTriggerCount</li>
 * 		<li> \b PCIE3126_TRIGGER_SINGLE_SEQUENCE : One whole sequence is read for each trigger</li>
 *	</ul>
 * </ul>
 * The external trigger is a combination of the following parameters :
 * <ul>
 * 	<li>The first one defines the trigger edge</li>
 *	<ul>
 *		<li> \b PCIE3126_EXT_TRIGGER_DISABLED : Trigger is disabled</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_LOW : Triggers on low edge</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_HIGH : Triggers on high edge</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_BOTH : Triggers on both edges</li>
 *	</ul>
 *	<li>The second one defines the trigger type</li>
	<ul>
 *		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
 *		<li> \b PCIE3126_TRIGGER_SEQUENCE_TRIGGER : A pre-defined number of sequence is acquired : dw_SequenceTriggerCount</li>
 *		<li> \b PCIE3126_TRIGGER_SINGLE_SEQUENCE : One whole sequence is read for each trigger</li>
 * 	</ul>
 * </ul>
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] dw_SequenceCount: The total number of sequence to acquire. Set it to 0 if you want to use continuous acquisition (i.e. infinite acquisition)
 * @param[in] dw_SequenceBeforeInterrupt: Specify the number of sequence to wait before generating an interruption. For high frequency, set a high number (512 if using maximum frequency for example).
 * @param[in] b_ChannelCount: The number of channels in one sequence. Rule: (b_ChannelCount * dw_SequenceBeforeInterrupt) must be a multiple of 2
 * @param[in] pb_Channel Define the channels in the sequence (single ended : 0 to 15, differential : 0 to 7). Depending on your board, this number may be different.
 * @param[in] pb_Gain Define the gain for each channel
 * @param[in] pb_Polarity Define the polarity for each channel (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
 * @param[in] dw_ConvertingTime Define the time between two channels acquisition (5us to 65 535 000us). If you choose 16 channels with a conversion time of 5us, an entire sequence will be acquired in 16 * 5 = 80us
 * @param[in] dw_DelayTime Define the time between two sequences acquisition (5us to 65 535 000us).
 * @param[in] b_SoftTriggerMode Defines the software trigger
 * @param[in] b_ExtTriggerMode Defines the external trigger
 * @param[in] dw_SequenceTriggerCount If you choose a trigger with mode PCIE3126_TRIGGER_SEQUENCE_TRIGGER, it defines the number of sequence to acquire when a trigger occurs (1 to 0xFFFFFFFF). If not use, give 0.
 * @return		  0 : success \n
 *				 -1 : The device handle parameter is wrong \n
 *		    	 -2 : The sequence size is wrong \n
 *				 -3 : The input mode has not been set \n
 *				 -4 : A channel index is invalid (single-ended mode) \n
 *				 -5 : A gain value is wrong \n
 *				 -6 : A polarity value is wrong \n
 *				 -7 : The converting time is invalid \n
 *				 -8 : The delay time is invalid \n
 *				 -9 : The interrupt routine is not installed \n
 *				-10 : dw_SequenceBeforeInterrupt is not valid \n
 *				-11 : (b_ChannelCount * dw_SequenceBeforeInterrupt) is not a multiple of 2 \n
 *				-12 : both software and hardware trigger are enabled. In that case, they must have the same type (one shot, sequence trigger or single sequence) \n
 *				-13 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_InitAndStartAnalogInputSequence(HANDLE	h_DeviceHandle,
												DWORD	dw_SequenceCount,
												DWORD	dw_SequenceBeforeInterrupt,
												BYTE	b_ChannelCount,
												PBYTE	pb_Channel,
												PBYTE	pb_Gain,
												PBYTE	pb_Polarity,
												DWORD	dw_ConvertingTime,
												DWORD	dw_DelayTime,
												BYTE	b_SoftTriggerMode,
												BYTE	b_ExtTriggerMode,
												DWORD	dw_SequenceTriggerCount);

 /** Read analog inputs from the board \n
 * You must call the set analog input mode function before calling this function. \n
 * Gain acts as a multiplier for the input voltage. Possible gain values are :
 * <ul>
 * 	<li> \b PCIE3126_GAIN_1 : Input range is between -10V and 10V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_2 : Input range is between -5V and 5V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_5 : Input range is between -2V and 2V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_10 : Input range is between -1V and 1V (bipolar)</li>
 * </ul>
 * The converting time stands for the time spent to acquire one channel. The time to acquire a sequence is ul_ConvertingTime * uc_ChannelCount \n
 * The software trigger is a combination of 2 parameters
 * <ul>
 * 	<li>The first one enables or disabled the trigger</li>
 *	<ul>
 *	 	<li> \b PCIE3126_SOFT_TRIGGER_ENABLED : The trigger is enabled</li>
 *		<li> \b PCIE3126_SOFT_TRIGGER_DISABLED : The trigger is disabled</li>
 *	</ul>
 * 	<li>The trigger type is selected with one of the following parameters</li>
 * 	<ul>
 *		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
 *	</ul>
 * </ul>
 * The external trigger is a combination of the following parameters :
 * <ul>
 * 	<li>The first one defines the trigger edge</li>
 *	<ul>
 *		<li> \b PCIE3126_EXT_TRIGGER_DISABLED : Trigger is disabled</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_LOW : Triggers on low edge</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_HIGH : Triggers on high edge</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_BOTH : Triggers on both edges</li>
 *	</ul>
 *	<li>The second one defines the trigger type</li>
	<ul>
 *		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
 * 	</ul>
 * </ul>
 * If the interrupts are enabled, the function returns, and the value can be read in the interrupt routine once available. \n
 * If the interrupts are disabled, the function waits for the results and return the values in the pus_ChannelValue parameter. \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_ChannelCount Set the number of channels in a sequence (1 to 16)
 * @param[in] pb_Channel Define the channels in the sequence (single ended : 0 to 15, differential : 0 to 7)
 * @param[in] pb_Gain Define the gain for each channel
 * @param[in] pb_Polarity Define the polarity for each channel (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
 * @param[in] dw_ConvertingTime Define the tine between two channels acquisition (5us to 65 535 000us)
 * @param[in] b_InterruptFlag Enable the interrupts
 * @param[in] b_SoftTriggerMode Defines the software trigger
 * @param[in] b_ExtTriggerMode Defines the external trigger
 * @param[out] pw_ChannelValue Value of the sequence, if interrupts are disabled
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : The sequence size is wrong \n
 *			-3 : The input mode has not been set \n
 *			-4 : A channel index is invalid (single-ended mode) \n
 *			-5 : A channel index is invalid (differential mode) \n
 *			-6 : A gain value is wrong \n
 *			-7 : A polarity value is wrong \n
 *			-8 : The converting time is invalid \n
 *			-9 : The interrupt routine is not installed \n
 *			-10 : The software trigger is invalid \n
 *			-11 : The external trigger is invalid \n
 *			-12 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_ReadMoreAnalogInputs (HANDLE	h_DeviceHandle,
									  BYTE		b_ChannelCount,
									  PBYTE		pb_Channel,
									  PBYTE		pb_Gain,
									  PBYTE		pb_Polarity,
									  DWORD		dw_ConvertingTime,
									  BYTE		b_InterruptFlag,
									  BYTE		b_SoftTriggerMode,
									  BYTE		b_ExtTriggerMode,
									  PWORD		pw_ChannelValue );

  /** Read one analog input from the board \n
 * You must call the set analog input mode function before calling this function. \n
 * Gain acts as a multiplier for the input voltage. Possible gain values are :
 * <ul>
 * 	<li> \b PCIE3126_GAIN_1 : Input range is between -10V and 10V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_2 : Input range is between -5V and 5V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_5 : Input range is between -2V and 2V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_10 : Input range is between -1V and 1V (bipolar)</li>
 * </ul>
 * The converting time stands for the time spent to acquire one channel. The time to acquire a sequence is ul_ConvertingTime * uc_ChannelCount \n
 * The software trigger is a combination of 2 parameters
 * <ul>
 * 	<li>The first one enables or disabled the trigger</li>
 *	<ul>
 *	 	<li> \b PCIE3126_SOFT_TRIGGER_ENABLED : The trigger is enabled</li>
 *		<li> \b PCIE3126_SOFT_TRIGGER_DISABLED : The trigger is disabled</li>
 *	</ul>
 * 	<li>The trigger type is selected with one of the following parameters</li>
 * 	<ul>
 *		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
 *	</ul>
 * </ul>
 * The external trigger is a combination of the following parameters :
 * <ul>
 * 	<li>The first one defines the trigger edge</li>
 *	<ul>
 *		<li> \b PCIE3126_EXT_TRIGGER_DISABLED : Trigger is disabled</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_LOW : Triggers on low edge</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_HIGH : Triggers on high edge</li>
 *		<li> \b PCIE3126_EXT_TRIGGER_ENABLED_BOTH : Triggers on both edges</li>
 *	</ul>
 *	<li>The second one defines the trigger type</li>
	<ul>
 *		<li> \b PCIE3126_TRIGGER_ONE_SHOT : The acquisition starts when the trigger is set</li>
 * 	</ul>
 * </ul>
 * If the interrupts are enabled, the function returns, and the value can be read in the interrupt routine once available. \n
 * If the interrupts are disabled, the function waits for the results and return the values in the pus_ChannelValue parameter. \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel Define the channel to acquire (single ended : 0 to 15, differential : 0 to 7)
 * @param[in] b_Gain Define the gain for the channel
 * @param[in] b_Polarity Define the polarity for the channel (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
 * @param[in] dw_ConvertingTime Define the time needed to acquire the channel (5us to 65 535 000us)
 * @param[in] b_InterruptFlag Enable the interrupts
 * @param[in] b_SoftTriggerMode Defines the software trigger
 * @param[in] b_ExtTriggerMode Defines the external trigger
 * @param[out] pw_ChannelValue Value of the channel, if interrupts are disabled
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-3 : The input mode has not been set \n
 *			-4 : A channel index is invalid (single-ended mode) \n
 *			-5 : A channel index is invalid (differential mode) \n
 *			-6 : A gain value is wrong \n
 *			-7 : A polarity value is wrong \n
 *			-8 : The converting time is invalid \n
 *			-9 : The interrupt routine is not installed \n
 *			-10 : The software trigger is invalid \n
 *			-11 : The external trigger is invalid \n
 *			-12 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Read1AnalogInput (HANDLE	h_DeviceHandle,
								  BYTE		b_Channel,
								  BYTE		b_Gain,
								  BYTE		b_Polarity,
								  DWORD		dw_ConvertingTime,
								  BYTE		b_InterruptFlag,
								  BYTE		b_SoftTriggerMode,
								  BYTE		b_ExtTriggerMode,
								  PWORD		pw_ChannelValue );

 /** Starts an auto-refresh acquisition \n
 * You must call the set analog input mode function before calling this function. \n
 * Auto-refresh mode is started once, and returns an array of values, each one corresponding to a channel. \n
 * These values are automatically refreshed after each conversion cycle, so the values in this array are always the latest ones. \n
 * Gain acts as a multiplier for the input voltage. Possible gain values are :
 * <ul>
 * 	<li> \b PCIE3126_GAIN_1 : Input range is between -10V and 10V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_2 : Input range is between -5V and 5V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_5 : Input range is between -2V and 2V (bipolar)</li>
 * 	<li> \b PCIE3126_GAIN_10 : Input range is between -1V and 1V (bipolar)</li>
 * </ul>
 * The converting time stands for the time spent to acquire one channel. The time to acquire a sequence is dw_ConvertingTime * b_ChannelCount \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_ChannelCount Set the number of channels in a sequence (1 to 16)
 * @param[in] pb_Channel Define the channels in the sequence (single ended : 0 to 15, differential : 0 to 7)
 * @param[in] pb_Gain Define the gain for each channel
 * @param[in] pb_Polarity Define the polarity for each channel (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
 * @param[in] dw_ConvertingTime Define the tine between two channels acquisition (5us to 65 535 000us)
 * @param[out] ppdw_Channels Pointer to an array that contains the value of each channels
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : The sequence size is wrong \n
 *			-3 : The input mode has not been set \n
 *			-4 : A channel index is invalid (single-ended mode) \n
 *			-5 : A channel index is invalid (differential mode) \n
 *			-6 : A gain value is wrong \n
 *			-7 : A polarity value is wrong \n
 *			-8 : The converting time is invalid \n
 *			-9 : Error while calling IOCTL function (wrong handle ?) \n
 *		   -20 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_InitAndStartAutoRefreshAcquisition(HANDLE	h_DeviceHandle,
								 BYTE	b_ChannelCount,
								 PBYTE	pb_Channel,
								 PBYTE	pb_Gain,
								 PBYTE	pb_Polarity,
								 DWORD	dw_ConvertingTime,
								 PDWORD	*ppdw_Channels );
	
/** Stops any analog acquisition \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_StopAnalogAcquisition(HANDLE	h_DeviceHandle );

/** Triggers any analog acquisition \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_TriggerAnalogAcquisition(HANDLE	h_DeviceHandle );

/** Get the number of analog inputs available on this board
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_NumberOfInputChannels Number of analog inputs available on this board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 */
_INT_ i_PCIe3126_GetAnalogInputChannelsNbr(HANDLE	h_DeviceHandle,
										 PBYTE	pb_NumberOfInputChannels );

/** Get the number of analog outputs available on this board
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_NumberOfOutputChannels Number of analog outputs available on this board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 */
_INT_ i_PCIe3126_GetAnalogOutputChannelsNbr(HANDLE	h_DeviceHandle,
										 PBYTE	pb_NumberOfOutputChannels );

/** Enables to read data acquired by a sequence acquisition. \n
 *
 * This function can be called in the interrupt routine that you registered. (see samples) \n
 *
 * @param[in]  h_DeviceHandle Handle of the board
 * @param[in]  dw_Size Size of data to read. To be powerfull, should be ( sequence size * sequence before interrupt ) * sizeof ( USHORT )
 * @param[out] pv_Data Pointer on a buffer where the data will be copied to
 * @return	>= 0:	size of data read \n
 *			-1	:	error while reading
 */
_LONG_ l_PCIe3126_ReadSequenceData(HANDLE h_DeviceHandle,
						 DWORD dw_Size,
						 PVOID pv_Data);
									 
/** Set the value of one analog output
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] uc_Channel Define the channel to set
 * @param[in] uc_Polarity Define the polarity for the channel (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
 * @param[in] us_Value Value to be set on the analog output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-3 : Wrong channel \n
 *			-4 : Wrong polarity \n
 *			-5 : Error while calling IOCTL function (wrong handle ?) \n
 */
_INT_ i_PCIe3126_Write1AnalogOutput (
	HANDLE	h_DeviceHandle,
	UCHAR	uc_Channel, 
	UCHAR	uc_Polarity,
	USHORT	us_Value );
 
 /** Set the value of several analog outputs
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] uc_ChannelNbr Define the number of channels to set
 * @param[in] puc_Channel Pointer to an array containing the list of channels to be set
 * @param[in] puc_Polarity Pointer to an array containing the list of polarities for the defined channels (PCIE3126_OUTPUT_MODE_UNIPOLAR or PCIE3126_OUTPUT_MODE_BIPOLAR)
 * @param[in] pus_Value Pointer to an array containing the values to be set
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : Wrong channel count \n
 *			-3 : Wrong channel \n
 *			-4 : Wrong polarity \n
 *			-5 : Error while calling IOCTL function (wrong handle ?) \n
 */
 _INT_ i_PCIe3126_WriteMoreAnalogOutputs(
	HANDLE	h_DeviceHandle, 
	UCHAR	uc_ChannelNbr, 
	PUCHAR	puc_Channel, 
	PUCHAR	puc_Polarity,
	PUSHORT	pus_Value );

/** Retrieves the analog input software and hardware trigger status
*
* @param[in]  h_DeviceHandle Handle of the board
* @param[out] softwareTriggerStatus Pointer to an unsingned long containing:
* <ul>
*	<li>in bit 0 if the trigger is enabled or disabled (ADDIDATA_ENABLE or ADDIDATA_DISABLE)</li>
*	<li>in bit 1 the status of the trigger (0 software trigger did not occur, 1 software trigger occurred)</li>
* </ul>
* @param[out] hardwareTriggerStatus Pointer to an unsigned long containing:
* <ul>
*	<li>in bit 0 if the trigger is enabled or disabled (ADDIDATA_ENABLE or ADDIDATA_DISABLE)</li>
*	<li>in bit 1 the trigger status</li>
*	<li>in bit 2 the trigger state</li>
* </ul>
* @param[out] hardwareTriggerCount Pointer to un unsigned long containing the hardware trigger count.
*/
_INT_ i_PCIe3126_GetAnalogInputTriggerStatus(
	HANDLE h_DeviceHandle,
	PULONG softwareTriggerStatus,
	PULONG hardwareTriggerStatus,
	PULONG hardwareTriggerCount);

/** Sets a bit to 1 in a global variable to release a blocked polling reading in case of hardware trigger disabling.
*
* @param[in]  h_DeviceHandle Handle of the board
*/
_INT_ i_PCIe3126_ReleaseHardwareTriggerWaitingRead(HANDLE h_DeviceHandle);

/** Sets a bit to 1 in a global variable to release a blocked polling reading in case of software trigger disabling.
*
* @param[in]  h_DeviceHandle Handle of the board
*/
_INT_ i_PCIe3126_ReleaseSoftwareTriggerWaitingRead(HANDLE h_DeviceHandle);


/** Computes convertion time given Convertion Time Unit and returns a result in micro seconds.
* @param[in,out]	pdw_ConvertionTime the time to convert
* @param[in]		b_ConvertionTimeUnit the time unit to use (0 for ns, 1 for micro s, 2 for ms, 3 for s)
*/
_INT_ i_PCIe3126_GetConvertionTimeInMicroSeconds(DWORD* pdw_ConvertionTime, BYTE b_ConvertionTimeUnit);

/**
 * Reads the value of a digital potentiometer (for calibration purposes)
 * @param h_DeviceHandle: handle to the board
 * @param b_PotisId: the ID number of the potentiometer to be read
 * <ul>
 * 	<li> \b 0 : Offset poti bipolar </li>
 * 	<li> \b 1 : Gain poti bipolar </li>
 * 	<li> \b 2 : Offset poti unipolar </li>
 * 	<li> \b 3 : Analog output gain </li>
 * </ul>
 * @param pdw_Value: A pointer to a value that will be filled with the read value
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : The potis id parameter is wrong
 * @retval -3 : Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCIe3126_ReadPotis(HANDLE h_DeviceHandle, BYTE b_PotisId, PDWORD pdw_Value);

/**
 * Writes the value of a digital potentiometer (for calibration purposes)
 * @param h_DeviceHandle: handle to the board
 * @param b_PotisId: the ID number of the potentiometer to be written
 * <ul>
 * 	<li> \b 0 : Offset poti bipolar </li>
 * 	<li> \b 1 : Gain poti bipolar </li>
 * 	<li> \b 2 : Offset poti unipolar </li>
 * 	<li> \b 3 : Analog output gain </li>
 * </ul>
 * @param dw_Value: The value to be written (0-255)
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : The potis id parameter is wrong
 * @retval -3 : The value parameter is wrong
 * @retval -4 : Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCIe3126_WritePotis(HANDLE h_DeviceHandle, BYTE b_PotisId, DWORD dw_Value);

/**
 * Save the values of the digital potentiometers in the EEPROM (for calibration purposes)
 * @param h_DeviceHandle: handle to the board
 * @retval 0 : No Error
 * @retval -1 : The device handle parameter is wrong
 * @retval -2 : Error while calling IOCTL function (wrong handle ?)
**/
_INT_ i_PCIe3126_SavePotis(HANDLE h_DeviceHandle);

/**
 *		@}
 *	@}
 */


						 
#ifdef __cplusplus
	}
#endif
