using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ZedGraph;

namespace analog_input
{
    public partial class gui : Form
    {
        // ----------------- Attributes -----------------
        //-----------------------------------------------
        /// <summary>
        /// Define the number of channels
        /// </summary>
        private const int channelNumber = 16;

        // Object pcie3126
        private pcie3126 pcie3126;

        // PCIe3126 related variables
        //---------------------------
        UInt32 sequenceCount           = 0; // 0 is continuous
		UInt32 sequenceBeforeInterrupt = 1024;
		byte channelCount              = 2;
        byte[] channelArray            ;
        byte[] gainArray               ;
        byte[] polarityArray           ;
		UInt32 convertingTime          = 5; // in us
		byte delayTime                 = 0;
		byte softTriggerMode           = 0;
		byte extTriggerMode            = 0;
		byte sequenceTriggerCount      = 0;
        byte inputMode                 = 2;  // 2 is diff

        /// <summary>
        /// Array that will store the channel value (computed)
        /// </summary>
        private double[] channelValues;

        // GUI related variables
        //----------------------

        /// <summary>
        /// Useful variable that enables us to make easy access on the channels boxes
        /// </summary>
        private CheckBox[] channelsBox;

        // Graph related variables
        //------------------------
        String[] channelTitle;

        /// <summary>
        /// Color used for the chart
        /// </summary>
        private Color[] colors = { Color.Red, Color.Blue, Color.Green, Color.Black, 
                                   Color.Purple, Color.DarkOrange, Color.Magenta, Color.Maroon,
                                   Color.Pink, Color.LightCyan, Color.Goldenrod, Color.DarkKhaki,
                                   Color.DarkRed, Color.DarkViolet, Color.Indigo, Color.Yellow};

        /// <summary>
        /// Fill of colors, only created once
        /// </summary>
        //private Fill colorsFill;

        // ----------------- Methods -----------------
        //--------------------------------------------

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public gui()
        {
            InitializeComponent();

            // Create the useful array
            this.channelsBox = new CheckBox[16];
            this.channelsBox[0] = this.channel0;
            this.channelsBox[1] = this.channel1;
            this.channelsBox[2] = this.channel2;
            this.channelsBox[3] = this.channel3;
            this.channelsBox[4] = this.channel4;
            this.channelsBox[5] = this.channel5;
            this.channelsBox[6] = this.channel6;
            this.channelsBox[7] = this.channel7;
            this.channelsBox[8] = this.channel8;
            this.channelsBox[9] = this.channel9;
            this.channelsBox[10] = this.channel10;
            this.channelsBox[11] = this.channel11;
            this.channelsBox[12] = this.channel12;
            this.channelsBox[13] = this.channel13;
            this.channelsBox[14] = this.channel14;
            this.channelsBox[15] = this.channel15;

            this.channel3.Enabled = false;
            this.channel4.Enabled = false;
            this.channel5.Enabled = false;
            this.channel6.Enabled = false;
            this.channel7.Enabled = false;
            this.channel8.Enabled = false;
            this.channel9.Enabled = false;
            this.channel10.Enabled = false;
            this.channel11.Enabled = false;
            this.channel12.Enabled = false;
            this.channel13.Enabled = false;
            this.channel14.Enabled = false;
            this.channel15.Enabled = false;            
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void gui_Load(object sender, EventArgs e)
        {      
            this.InitializeChartGraphics();
        }

        /// <summary>
        /// Initialize the chart graphics
        /// </summary>
        private void InitializeChartGraphics()
        {
            this.chartControl.GraphPane.XAxis.Title.Text = "Time (us)";
            this.chartControl.GraphPane.YAxis.Title.Text = "Value (V)";
        }


        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void gui_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Stop the sequence if any is running
            if (startButton.Enabled == false)
            {
                this.pcie3126.ReleaseAcquisition();
            }
        }

        /// <summary>
        /// Click on select / deselect check - box
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectDeselectBox_CheckedChanged(object sender, EventArgs e)
        {
            // Select - deselect all
            for (int i = 0; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = this.selectDeselectBox.Checked;
            }
        }

        /// <summary>
        /// Return the number of selected channels
        /// </summary>
        /// <returns></returns>
        private byte getSelectChannelsCount()
        {
            byte count = 0;
            for (int i = 0; i < channelsBox.Length; i++)
            {
                if (channelsBox[i].Checked)
                {
                    count++;
                }
            }
            return count;
        }

        /// <summary>
        /// Check and return the correct value of the sequence before interrupt parameter
        /// </summary>
        /// <param name="sequenceBeforeInterrupt"></param>
        /// <returns></returns>
        private bool checkAndGetSequenceBeforeInterruptCount(ref uint sequenceBeforeInterrupt)
        {
            try
            {
                sequenceBeforeInterrupt = Convert.ToUInt32(this.sequenceBeforeInterruptText.Text);
                if (sequenceBeforeInterrupt == 0)
                {
                    return false;
                }
                return true;
            }
            catch (Exception)
            {
                sequenceBeforeInterrupt = 0;
                return false;
            }
        }

        /// <summary>
        /// Check and return the correct value of the converting time parameter
        /// </summary>
        /// <param name="convertingTime"></param>
        /// <returns></returns>
        private bool checkAndGetConvertingTime(ref uint convertingTime)
        {
            try
            {
                convertingTime = Convert.ToUInt32(this.convertingTimeText.Text);
                if ( (convertingTime < 5) || (convertingTime > 65535000 ) )
                {
                    convertingTime = 0;
                    return false;
                }
                return true;
            }
            catch (Exception)
            {
                convertingTime = 0;
                return false;
            }
        }

        /// <summary>
        /// Construct the channel array
        /// </summary>
        /// <returns></returns>
        private byte[] getChannelArray()
        {
            int currentIndex = 0;
            byte[] array = new byte[channelNumber];

            for (int i = 0; i < gui.channelNumber; i++)
            {
                array[i] = 0;
                if (this.channelsBox[i].Checked == true)
                {
                    array[currentIndex] = (byte)i;
                    currentIndex++;
                }
            }
            return array;
        }

        /// <summary>
        /// Click on the start button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startButton_Click(object sender, EventArgs e)
        {
            int ret = 0;
            displayTimer.Interval = Convert.ToInt32(displayUpdateInMsTextBox.Text);

            // Check channel selection
            if ((channelCount = getSelectChannelsCount()) == 0)
            {
                MessageBox.Show("Select at least one channel");
                return;
            }

            // Check and set the analog input mode
            if ((seRadio.Checked == false) && (diffRadio.Checked == false))
            {
                MessageBox.Show("Please choose the Input mode, SE or DIFF");
                return;
            }
            if (seRadio.Checked == true)
                inputMode = 1;
            else
                inputMode = 2;

            if (this.checkAndGetSequenceBeforeInterruptCount(ref sequenceBeforeInterrupt) == false)
            {
                MessageBox.Show("Sequence before interrupt : invalid value");
                return;
            }

            if (this.checkAndGetConvertingTime(ref convertingTime) == false)
            {
                MessageBox.Show("Converting time : invalid value");
                return;
            }

            // Construct the gain and polarity array
            gainArray     = new byte[gui.channelNumber];
            polarityArray = new byte[gui.channelNumber];
            for (int i = 0; i < gui.channelNumber; i++)
            {
                gainArray[i]     = 0; /* 0 is gain 1*/
                polarityArray[i] = 1; /* 1 is bipolar */
            }

            // Construct the channel array
            this.channelArray = getChannelArray();

            // Start the chart now that we have the whole configuration
            this.startChart();
            
            // Construct object pcie3126
            pcie3126 = new pcie3126();

            // Start the sequence acquisition
            ret = pcie3126.InitAndStartAcquisition(sequenceCount          ,
			                                       sequenceBeforeInterrupt,
			                                       channelCount           ,
			                                       channelArray           ,
                                                   gainArray              ,
                                                   polarityArray          ,
                                                   convertingTime         ,
                                                   delayTime              ,
                                                   softTriggerMode        ,
                                                   extTriggerMode         ,
                                                   sequenceTriggerCount,
                                                   inputMode);	

            if (ret != 0)
            {
                MessageBox.Show("Error starting acquisition : " + ret.ToString());
                return;
            }

            // Update the GUI
            this.startButton.Enabled = false;
            
            this.displayTimer.Start();

        }


        /// <summary>
        /// Start the chart
        /// </summary>
        private void startChart()
        {                    
            this.channelTitle  = new String[this.channelCount];
            this.channelValues = new double[this.channelCount];
            
            this.chartControl.GraphPane.CurveList.Clear();
       
            // Fill curve title name
            for (int i = 0; i < channelArray.Length; i++)
            {
                if (this.channelArray[i] == i)
                {
                    this.channelTitle[i] = "Ch. " + channelArray[i].ToString();                    
                }
            }

            // Set the title line      
            this.chartControl.Invalidate();
            this.chartControl.AxisChange();            
        }

        private void stop_all()
        {
            try
            {
                this.pcie3126.ReleaseAcquisition();
                this.pcie3126 = null;

                this.displayTimer.Stop();
                this.irqCountTextBox.Text = Convert.ToString(0);

                this.channelArray = null;
                this.gainArray = null;
                this.polarityArray = null;
                this.channelValues = null;
            }
            catch (Exception)
            {
                MessageBox.Show("No acquisition running");
            }

            this.startButton.Enabled = true;

        }

        /// <summary>
        /// Click on the stop button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopButton_Click(object sender, EventArgs e)
        {
            stop_all();
        }


        /// <summary>
        /// Display timer tick. Update the chart
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void displayTimer_Tick(object sender, EventArgs e)
        {
            int i, j;

            this.irqCountTextBox.Text = Convert.ToString(this.pcie3126.irqCount);

            if (!this.pcie3126.running)
            {
                stop_all();                
                MessageBox.Show("Error in the acquisition");
                return;
            }

            this.chartControl.GraphPane.CurveList.Clear();

            try
            {
                int readDataSize = pcie3126.readData.Length;  // also = this.sequenceBeforeInterrupt * channelCount
                // Fill the list array
                for (i = 0; i < this.channelCount; i++)
                {
                    PointPairList list = new PointPairList();
                    for (j = 0; j < readDataSize; j = j + this.channelCount)
                        list.Add(Convert.ToDouble(j * this.convertingTime + this.delayTime), this.pcie3126.readData[i + j]);
                    LineItem curve = this.chartControl.GraphPane.AddCurve(this.channelTitle[i], list, this.colors[i], SymbolType.Plus);
                }
            }
            catch (Exception)
            {
                stop_all();
                MessageBox.Show(string.Format("Error with Zgraph"));
                return;
            }
            this.chartControl.AxisChange();
            this.chartControl.Invalidate();
        }

        private void channel0_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 1; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }      

            if (this.channel0.Checked)
                this.channel1.Enabled = true;
            else
                this.channel1.Enabled = false;
        }

        private void channel1_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 2; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }   
      
            if (this.channel1.Checked)
                this.channel2.Enabled = true;
            else
                this.channel2.Enabled = false;
        }
        
        private void channel2_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 3; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel2.Checked)
                this.channel3.Enabled = true;
            else
                this.channel3.Enabled = false;
        }

        private void channel3_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 4; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }

            if (this.channel3.Checked)
                this.channel4.Enabled = true;
            else
                this.channel4.Enabled = false;
        }

        private void channel4_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 5; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }

            if (this.channel4.Checked)
                this.channel5.Enabled = true;
            else
                this.channel5.Enabled = false;
        }

        private void channel5_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 6; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }

            if (this.channel5.Checked)
                this.channel6.Enabled = true;
            else
                this.channel6.Enabled = false;
        }

        private void channel6_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 7; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel6.Checked)
                this.channel7.Enabled = true;
            else
                this.channel7.Enabled = false;
        }

        private void channel7_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 8; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel7.Checked)
                this.channel8.Enabled = true;
            else
                this.channel8.Enabled = false;
        }

        private void channel8_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 9; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel8.Checked)
                this.channel9.Enabled = true;
            else
                this.channel9.Enabled = false;
        }

        private void channel9_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 10; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }

            if (this.channel9.Checked)
                this.channel10.Enabled = true;
            else
                this.channel10.Enabled = false;
        }

        private void channel10_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 11; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel0.Checked)
                this.channel11.Enabled = true;
            else
                this.channel11.Enabled = false;
        }

        private void channel11_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 12; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }

            if (this.channel11.Checked)
                this.channel12.Enabled = true;
            else
                this.channel12.Enabled = false;
        }

        private void channel12_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 13; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel12.Checked)
                this.channel13.Enabled = true;
            else
                this.channel13.Enabled = false;
        }

        private void channel13_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 14; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel13.Checked)
                this.channel14.Enabled = true;
            else
                this.channel14.Enabled = false;
        }

        private void channel14_CheckedChanged(object sender, EventArgs e)
        {
            for (int i = 15; i < gui.channelNumber; i++)
            {
                this.channelsBox[i].Checked = false;
                this.channelsBox[i].Enabled = false;
            }
            
            if (this.channel14.Checked)
                this.channel15.Enabled = true;
            else
                this.channel15.Enabled = false;
        }


    }
}