﻿using System;
using System.Diagnostics;
using System.Windows.Forms;
using System.IO;
using System.Text;

namespace analog_input
{
	
    public class pcie3126
    {
        // ----------------- Attributes -----------------
        public const byte NB_CHANNELS = 16;
        public const byte PCIE3126_NO_DELAY = 0;
        public const byte PCIE3126_SOFT_TRIGGER_DISABLED = 0;
        public const byte PCIE3126_EXT_TRIGGER_DISABLED  = 0;
        public const byte PCIE3126_GAIN_1  = 0;
        public const byte PCIE3126_GAIN_2  = 1;
        public const byte PCIE3126_GAIN_5  = 2;
        public const byte PCIE3126_GAIN_10 = 3;

        public bool running = true;

        /// <summary>
        /// Board object
        /// </summary>
        public static PCIe3126_NET.PCIe3126 board;
       
        private UInt32 sequenceCount           = 512;
        private UInt32 sequenceBeforeInterrupt = 512;
        private byte channelCount              = 0;
        private byte[] channel;
        private byte[] gain;
        private byte[] polarity;
        private UInt32 convertingTime          = 0;
        private byte delayTime                 = PCIE3126_NO_DELAY;
        private byte softTriggerMode           = PCIE3126_SOFT_TRIGGER_DISABLED;
        private byte extTriggerMode            = PCIE3126_EXT_TRIGGER_DISABLED;
		private byte sequenceTriggerCount      = 0;

        //public StreamWriter fs;
        //string path = @"log.txt";

        public UInt32 irqCount       = 0;

        public double[] readData = null;

        // ----------------- Methods -----------------
        /// <summary>
        /// Constructor
        /// </summary>>
        public pcie3126()
        {
			this.sequenceCount           = 512;
			this.sequenceBeforeInterrupt = 512;
			this.channelCount            = 0;

            this.channel  = new byte[NB_CHANNELS];
			this.gain     = new byte[NB_CHANNELS];
			this.polarity = new byte[NB_CHANNELS];

			this.convertingTime          = 0;
			this.delayTime               = PCIE3126_NO_DELAY;
			this.softTriggerMode         = PCIE3126_SOFT_TRIGGER_DISABLED;
			this.extTriggerMode          = PCIE3126_EXT_TRIGGER_DISABLED;
			this.sequenceTriggerCount    = 0;

            this.irqCount            = 0;

            // Create a file, or overwrite if the file exists.
            //using (this.fs = File.CreateText(path))
            //{
            //    fs.WriteLine("New file created: {0}", DateTime.Now.ToString());
            //}
        }

        /// <summary>
        /// Compute the voltage value according to the gain for bipolar mode
        /// </summary>
        double computeVoltage(ushort value, byte gain)
        {
            double scale;
            ushort aux = value;

            switch (gain)
            {
                case PCIE3126_GAIN_1:
                    scale = 10.0;
                    break;
                case PCIE3126_GAIN_2:
                    scale = 2.0;
                    break;
                case PCIE3126_GAIN_5:
                    scale = 5.0;
                    break;
                case PCIE3126_GAIN_10:
                    scale = 1.0;
                    break;
                default:
                    scale = 10.0;
                    break;
            }

            return ((double)(value - 32767)) * (scale / 32767.0);
        }

        void board_EventReceivedFifoOverflow(object source)
        {
            running = false;
            MessageBox.Show(string.Format("Received Event: FifoOverflow"));
        }

        void board_EventReceivedEndOfSequence(object source, PCIe3126_NET.EndOfSequenceEventArgs args)
        {
            running = false;
            MessageBox.Show(string.Format("Received End of Sequence"));
        }

        /// <summary>
        /// Interrupt received
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_EventReceived(object source, PCIe3126_NET.AnalogInputEventArgs args)
        {
            int ret = 0;
            bool error = false;
            ushort[] rawData = null;
            int inc = 0;

            irqCount++;
            if (irqCount >= 0xFFFFFFFE)
                irqCount = 0;

            // Append interrupt mask
            //using (fs = File.AppendText(path))
            //{
            //    fs.WriteLine("IRQ: {0}", args.InterruptMask);
            //}

            if ((args.InterruptMask & 0x4) == 0x4) 
            {
                MessageBox.Show(string.Format("DMA Fifo overflow: 0x{0:X}\n", args.InterruptMask));
                error = true;
            }

            if (args.FifoOverflow != 0)
            {                
                MessageBox.Show(string.Format("FifoOverflow: 0x{0:X}\n", args.FifoOverflow));
                error = true;
            }
            if ((args.ArgsSize <= 0))
            {                
                MessageBox.Show(string.Format("Issue with ArgsSize: 0x{0:X}\n", args.ArgsSize));
                error = true;
            }
            if (args.ArgsSize != sequenceBeforeInterrupt * channelCount)
            {                
                MessageBox.Show(string.Format("Issue with ArgsSize: 0x{0:X} different of sequenceBeforeInterrupt * channelCount ({})\n", args.ArgsSize, sequenceBeforeInterrupt * channelCount));
                error = true;
            }

            rawData = new ushort[sequenceBeforeInterrupt * channelCount];
            if (rawData == null)
            {
                MessageBox.Show(string.Format("Issue to allocate memory\n"));
                error = true;
            }

            if (error)
            {
                running = false;
                return;
            }

            ret = board.ReadSequenceData(args.ArgsSize * sizeof(ushort), rawData);
            if (ret < 0)
            {
                MessageBox.Show(string.Format("ReadSequenceData error: {}", ret));
                running = false;
                rawData = null;
                return;
            }
            for (int i = 0; i < this.readData.Length; i++)
            {
            //    // Compute the voltage value
            //    // Same gain for all, bipolar
                this.readData[i] = computeVoltage(rawData[i], gain[0]);
            //    //this.readData[i] = ((double)(rawData[i] - 32767)) * (10.0 / 32767.0);                
            }
            rawData = null;
        }

        /// <summary>
        /// Init and start acquisition
        /// </summary>
        public int InitAndStartAcquisition(UInt32 sequenceCount          ,
		                                   UInt32 sequenceBeforeInterrupt,
		                                   byte channelCount             ,
		                                   byte[] channel                ,
		                                   byte[] gain                   ,
		                                   byte[] polarity               ,
		                                   UInt32 convertingTime         ,
		                                   byte delayTime                ,
		                                   byte softTriggerMode          ,
										   byte extTriggerMode           ,
		                                   byte sequenceTriggerCount,
                                           byte inputMode)              
        {
            int ret = 0;
            byte i = 0;
            
            // Construct the board object
            board = PCIe3126_NET.PCIe3126.OpenBoard(0);
            if (board == null)
            {
                MessageBox.Show(string.Format("Issue while opening the board with index 0"));
                ret = -1;
                return ret;
            }

            readData = new double[sequenceBeforeInterrupt * channelCount];

            // Listen to the counter timer watchdog interruption
            board.AnalogInputEventReceived     += new PCIe3126_NET.PCIe3126.AnalogInputEventHandler(board_EventReceived);
            board.PldFifoOverflowEventReceived += new PCIe3126_NET.PCIe3126.PldFifoOverflowEventHandler(board_EventReceivedFifoOverflow);
            board.EndOfSequenceEventReceived   += new PCIe3126_NET.PCIe3126.EndOfSequenceEventHandler(board_EventReceivedEndOfSequence);

            // Copy arguments
            this.sequenceCount           = sequenceCount          ;
            this.sequenceBeforeInterrupt = sequenceBeforeInterrupt;
            this.channelCount            = channelCount           ;
			
			for (i = 0; i < NB_CHANNELS; i++)
			{
				this.channel[i]          = channel[i]             ;
				this.gain[i]             = gain[i]                ;
				this.polarity[i]         = polarity[i]            ;
			}
			
            this.convertingTime          = convertingTime         ;
            this.delayTime               = delayTime              ;
            this.softTriggerMode         = softTriggerMode        ;
			this.extTriggerMode          = extTriggerMode         ;
            this.sequenceTriggerCount    = sequenceTriggerCount   ;
            
            ret = board.SetBoardIntRoutine();
            if (ret != 0)
            {
                MessageBox.Show(string.Format("Error i_pcie3126_SetBoardIntRoutine: {0}\n", ret));
                ret = -100 + ret;
                return ret;
            }

            ret = board.SetAnalogInputMode(inputMode);
            if (ret != 0)
            {
                MessageBox.Show(string.Format("Error i_pcie3126_SetAnalogInputMode: {0}\n", ret));
                ret = -200 + ret;
                return ret;
            }

            //// Append parameters
            //using (fs = File.AppendText(path))
            //{
            //    fs.WriteLine("Parameters: convTime: {0}", this.convertingTime);
            //    fs.WriteLine("Parameters: delayTime: {0}", this.delayTime);
            //    fs.WriteLine("Parameters: sequenceBeforeInterrupt: {0}", this.sequenceBeforeInterrupt);
            //    fs.WriteLine("Parameters: channelCount: {0}", this.channelCount);
            //}

            ret = board.InitAndStartAnalogInputSequence(sequenceCount          ,
			                                            sequenceBeforeInterrupt,
			                                            channelCount           ,
			                                            channel                ,
			                                            gain                   ,
			                                            polarity               ,
			                                            convertingTime         ,
			                                            delayTime              ,
			                                            softTriggerMode        ,
			                                            extTriggerMode         ,
			                                            sequenceTriggerCount);			
            if (ret != 0)
            {
                MessageBox.Show(string.Format("Error i_PCIe3126_InitAndStartAnalogInputSequence: {0}\n", ret));
                ret = -300 + ret;
                return ret;
            }
            return 0;
        }

        public void ReleaseAcquisition()
        {           
            // cleanup
            board.StopAnalogAcquisition();
            board.ResetBoardIntRoutine();
            readData = null;
        }
    }
}