﻿using System;
using System.Diagnostics;

namespace analog_input
{
	
    public class pcie3126
    {
        // ----------------- Attributes -----------------
        public const byte NB_CHANNELS = 16;
        public const byte PCIE3126_NO_DELAY = 0;
        public const byte PCIE3126_SOFT_TRIGGER_DISABLED = 0;
        public const byte PCIE3126_EXT_TRIGGER_DISABLED = 0;  
        
        public static bool running = true;

        /// <summary>
        /// Board object
        /// </summary>
        public static PCIe3126_NET.PCIe3126 board;
       
        private UInt32 sequenceCount           = 512;
        private UInt32 sequenceBeforeInterrupt = 512;
        private byte channelCount              = 0;
        private byte[] channel;
        private byte[] gain;
        private byte[] polarity;
        private UInt32 convertingTime          = 0;
        private byte delayTime                 = PCIE3126_NO_DELAY;
        private byte softTriggerMode           = PCIE3126_SOFT_TRIGGER_DISABLED;
        private byte extTriggerMode            = PCIE3126_EXT_TRIGGER_DISABLED;
		private byte sequenceTriggerCount      = 0;
		

        private int irqCount       = 0;
        private int printCount     = 0;
        private int printFrequency = 30;

        // ----------------- Methods -----------------
        /// <summary>
        /// Constructor
        /// </summary>>
        public pcie3126()
        {
			this.sequenceCount           = 512;
			this.sequenceBeforeInterrupt = 512;
			this.channelCount            = 0;

            this.channel  = new byte[NB_CHANNELS];
			this.gain     = new byte[NB_CHANNELS];
			this.polarity = new byte[NB_CHANNELS];

			this.convertingTime          = 0;
			this.delayTime               = PCIE3126_NO_DELAY;
			this.softTriggerMode         = PCIE3126_SOFT_TRIGGER_DISABLED;
			this.extTriggerMode          = PCIE3126_EXT_TRIGGER_DISABLED;
			this.sequenceTriggerCount    = 0;

            this.irqCount            = 0;
            // Establish an event handler to process key press events.
            Console.CancelKeyPress += new ConsoleCancelEventHandler(CtrlHandler);
        }

        static void CtrlHandler(object sender, ConsoleCancelEventArgs args)
        {
            Console.WriteLine("\nCtrl+C pressed\n");
            running = false;
            // Set the Cancel property to true to prevent the process from terminating.
            //args.Cancel = true;
        }

        /// <summary>
        /// Interrupt received
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_EventReceived(object source, PCIe3126_NET.AnalogInputEventArgs args)
        {
            int ret = 0;
            bool error = false;
            uint i;
            ushort[] raw_data;
            
            printCount++;
            irqCount++;


            if (args.FifoOverflow != 0)
            {
                Console.WriteLine("FifoOverflow: 0x{0:X}\n", args.FifoOverflow);
                error = true;
            }
            if ((args.ArgsSize <= 0))
            {
                Console.WriteLine("Issue with ArgsSize: 0x{0:X}\n", args.ArgsSize);
                error = true;
            }

            if (printCount >= printFrequency || error)
            {
                Console.WriteLine("\nInterrupt #{0}:", irqCount);
                Console.WriteLine("  FifoOverflow   : 0x{0:X}", args.FifoOverflow);
                Console.WriteLine("  InterruptMask  : 0x{0:X}", args.InterruptMask);
                Console.WriteLine("  ArgsSize       : 0x{0:X}", args.ArgsSize);
            }

            if (error)
            {
                Console.WriteLine("Error\n");
                running = false;
                return;
            }

            raw_data = new ushort[sequenceBeforeInterrupt * channelCount];

            ret = board.ReadSequenceData(args.ArgsSize * sizeof(ushort), raw_data);
            if (ret < 0)
            {
                Console.WriteLine("ReadSequenceData error: " + ret.ToString());
                running = false;
                return;
            }

            if (printCount >= printFrequency)
            {
                Console.WriteLine("ReadSequenceData returns {0} Bytes", ret);

                // Displaying values
                for (i = 0; i < (uint)raw_data.Length && i < 10; i++)
                    Console.WriteLine("[{0:0000000000}] Values: 0x{1:X4}", i, raw_data[i]);
                Console.WriteLine("...");
                for (i = (uint)raw_data.Length - 10; i < (uint)raw_data.Length; i++)
                    Console.WriteLine("[{0:0000000000}] Values: 0x{1:X4}", i, raw_data[i]);

                Console.WriteLine("\n");
            } 

            if (printCount >= printFrequency)
                printCount = 0;
        }

        /// <summary>
        /// Init and start acquisition
        /// </summary>
        public int InitAndStartAcquisition(UInt32 sequenceCount          ,
		                                   UInt32 sequenceBeforeInterrupt,
		                                   byte channelCount             ,
		                                   byte[] channel                 ,
		                                   byte[] gain                     ,
		                                   byte[] polarity                 ,
		                                   UInt32 convertingTime         ,
		                                   byte delayTime                ,
		                                   byte softTriggerMode          ,
										   byte extTriggerMode           ,
		                                   byte sequenceTriggerCount,
                                           byte inputMode)              
        {
            int ret = 0;
            byte i = 0;

            // Construct the board object
            board = PCIe3126_NET.PCIe3126.OpenBoard(0);
            if (board == null)
            {
                Console.WriteLine("Issue while opening the board");
                ret = -1;
                goto end;
            }

            // Listen to the counter timer watchdog interruption
            board.AnalogInputEventReceived += new PCIe3126_NET.PCIe3126.AnalogInputEventHandler(board_EventReceived);

            // Copy arguments
            this.sequenceCount           = sequenceCount          ;
            this.sequenceBeforeInterrupt = sequenceBeforeInterrupt;
            this.channelCount            = channelCount           ;
			
			for (i = 0; i < NB_CHANNELS; i++)
			{
				this.channel[i]          = channel[i]             ;
				this.gain[i]             = gain[i]                ;
				this.polarity[i]         = polarity[i]            ;
			}
			
            this.convertingTime          = convertingTime         ;
            this.delayTime               = delayTime              ;
            this.softTriggerMode         = softTriggerMode        ;
			this.extTriggerMode          = extTriggerMode         ;
            this.sequenceTriggerCount    = sequenceTriggerCount   ;
            
            ret = board.SetBoardIntRoutine();
            if (ret != 0)
            {
                Console.WriteLine("Error i_pcie3126_SetBoardIntRoutine {0}\n", ret);
                ret = -1;
                goto end;
            }
            Console.WriteLine("i_pcie3126_SetBoardIntRoutine OK\n");

            ret = board.SetAnalogInputMode(inputMode);
            if (ret != 0)
            {
                Console.WriteLine("Error i_pcie3126_SetAnalogInputMode {0}\n", ret);
                ret = -1;
                goto end;
            }
            Console.WriteLine("i_pcie3126_SetAnalogInputMode OK\n");




            ret = board.InitAndStartAnalogInputSequence(sequenceCount          ,
			                                            sequenceBeforeInterrupt,
			                                            channelCount           ,
			                                            channel                ,
			                                            gain                   ,
			                                            polarity               ,
			                                            convertingTime         ,
			                                            delayTime              ,
			                                            softTriggerMode        ,
			                                            extTriggerMode         ,
			                                            sequenceTriggerCount);			
            if (ret != 0)
            {
                Console.WriteLine("i_PCIe3126_InitAndStartAnalogInputSequence Error : {0}\n", (int)ret);
                ret = -6;
                goto end;
            }
            Console.WriteLine("i_PCIe3126_InitAndStartAnalogInputSequence OK\n");

        end:
            return ret;
        }

        public void ReleaseAcquisition()
        {           
            // cleanup
            Console.WriteLine("Cleaning up\n");
            board.StopAnalogAcquisition();
            board.ResetBoardIntRoutine();
        }
    }
}