using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ReadMoreAnalogInputs
{
    public partial class ReadMoreAnalogInputs : Form
    {
        /// <summary>
        /// Define the number of channels
        /// </summary>
        private const int channelNumber = 16;

        /// <summary>
        /// Handle to the first board
        /// </summary>
        private PCIe3126_NET.PCIe3126 board;

        /// <summary>
        /// Useful variable that enables us to make easy access on the channels boxes
        /// </summary>
        private CheckBox[] channelsBox;

        /// <summary>
        /// List of selected channels
        /// </summary>
        byte[] channelArray;

        /// <summary>
        /// Delegate that enables to set the text of the text box from another thread
        /// </summary>
        /// <param name="text"></param>
        private delegate void SetTextDelegate(String text);

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public ReadMoreAnalogInputs()
        {
            InitializeComponent();

            // Create the useful array
            this.channelsBox = new CheckBox[channelNumber];
            this.channelsBox[0] = this.channel0;
            this.channelsBox[1] = this.channel1;
            this.channelsBox[2] = this.channel2;
            this.channelsBox[3] = this.channel3;
            this.channelsBox[4] = this.channel4;
            this.channelsBox[5] = this.channel5;
            this.channelsBox[6] = this.channel6;
            this.channelsBox[7] = this.channel7;
            this.channelsBox[8] = this.channel8;
            this.channelsBox[9] = this.channel9;
            this.channelsBox[10] = this.channel10;
            this.channelsBox[11] = this.channel11;
            this.channelsBox[12] = this.channel12;
            this.channelsBox[13] = this.channel13;
            this.channelsBox[14] = this.channel14;
            this.channelsBox[15] = this.channel15;
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReadMoreAnalogInputs_Load(object sender, EventArgs e)
        {
            int ret = 0;

            // Open the first board of the system
            this.board = PCIe3126_NET.PCIe3126.OpenBoard(0);
            // Error handling
            if (this.board == null)
            {
                MessageBox.Show("Error OpenBoard returned null");
                this.Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                this.Close();
                return;
            }

            // Add an event on the end of sequence interrupt
            this.board.EndOfSequenceEventReceived += new PCIe3126_NET.PCIe3126.EndOfSequenceEventHandler(board_EndOfSequenceEventReceived);
        }

        /// <summary>
        /// Set the current text of the result box
        /// Called by other thread generally to signal that read is finished
        /// </summary>
        /// <param name="text"></param>
        private void SetText(String text)
        {
            this.resultBox.Text = text;

            // Stop the progress bar
            this.progressBar.Style = ProgressBarStyle.Blocks;

            // Enable to read again
            this.readButton.Enabled = true;
        }

        /// <summary>
        /// Get the end of sequence interrupt
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_EndOfSequenceEventReceived(object source, PCIe3126_NET.EndOfSequenceEventArgs args)
        {
            String text = "Interrupt \r\n";

            for (int i = 0; i < args.ArgsSize; i++)
            {
                text += "Channel " + this.channelArray[i].ToString() + " = " + args.Args[i] + "\r\n";
            }

            this.Invoke(new SetTextDelegate(this.SetText), text);
        }

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ReadMoreAnalogInputs_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;
            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }

        /// <summary>
        /// Select / Deselect all the channels
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void selectDeselectBox_CheckedChanged(object sender, EventArgs e)
        {
            // Select - deselect all
            for (int i = 0; i < ReadMoreAnalogInputs.channelNumber; i++)
            {
                this.channelsBox[i].Checked = this.selectDeselectBox.Checked;
            }
        }

        /// <summary>
        /// Check and return the correct value of the converting time parameter
        /// </summary>
        /// <param name="convertingTime"></param>
        /// <returns></returns>
        private bool checkAndGetConvertingTime(ref uint convertingTime)
        {
            try
            {
                convertingTime = Convert.ToUInt32(this.convertingTimeText.Text);
                if ((convertingTime < 5) || (convertingTime > 65535000))
                {
                    convertingTime = 0;
                    return false;
                }
                return true;
            }
            catch (Exception)
            {
                convertingTime = 0;
                return false;
            }
        }

        /// <summary>
        /// Return the number of selected channels
        /// </summary>
        /// <returns></returns>
        private uint getSelectChannelsCount()
        {
            uint count = 0;
            for (int i = 0; i < channelsBox.Length; i++)
            {
                if (channelsBox[i].Checked)
                {
                    count++;
                }
            }
            return count;
        }

        /// <summary>
        /// Construct the channel array
        /// </summary>
        /// <returns></returns>
        private byte[] getChannelArray()
        {
            int currentIndex = 0;
            byte[] array = new byte[getSelectChannelsCount()];
            for (int i = 0; i < ReadMoreAnalogInputs.channelNumber; i++)
            {
                if (this.channelsBox[i].Checked == true)
                {
                    array[currentIndex] = (byte)i;
                    currentIndex++;
                }
            }
            return array;
        }

        /// <summary>
        /// Click on the read button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readButton_Click(object sender, EventArgs e)
        {
            uint convertingTime = 0;
            byte[] gainArray;
            byte[] polarityArray;
            int returnValue = 0;
            uint channelCount = getSelectChannelsCount();

            this.resultBox.Text = "";
            this.resultBox.Update();

            if (channelCount == 0)
            {
                MessageBox.Show("Select at least one channel");
                return;
            }

            // Check the analog input mode
            if ((seRadio.Checked == false) && (diffRadio.Checked == false))
            {
                MessageBox.Show("Please choose the input mode");
                return;
            }

            // Check the interrupt mode
            if ((yesRadio.Checked == false) && (noRadio.Checked == false))
            {
                MessageBox.Show("Please choose the interrupt mode");
                return;
            }

            if (this.checkAndGetConvertingTime(ref convertingTime) == false)
            {
                MessageBox.Show("Converting time : invalid value");
                return;
            }

            // Construct the gain and polarity array
            gainArray = new byte[ReadMoreAnalogInputs.channelNumber];
            polarityArray = new byte[ReadMoreAnalogInputs.channelNumber];
            for (int i = 0; i < ReadMoreAnalogInputs.channelNumber; i++)
            {
                gainArray[i] = 0; /* gain 1*/
                polarityArray[i] = 1; /* bipolar */
            }

            // Construct the channel array
            this.channelArray = getChannelArray();

            // Start progress bar and disable read button
            this.progressBar.Style = ProgressBarStyle.Marquee;
            this.readButton.Enabled = false;

            // Now two cases, interrupt or not ...
            if (this.yesRadio.Checked == true)
            {
                // Interrupt
                returnValue = this.board.ReadMoreAnalogInputs((byte)channelCount,
                    channelArray,
                    gainArray,
                    polarityArray,
                    convertingTime,
                    1,      // interrupt
                    0,      // no software trigger
                    0,      // no hardware trigger
                    null);

                if (returnValue != 0)
                {
                    // Error, set the text
                    this.SetText("ReadMoreAnalogInputs error : " + returnValue.ToString());
                    return;
                }
            }
            else
            {
                // No interrupt
                // Use a background worker to not block the graphic interface
                Configuration configuration = new Configuration();
                configuration.channelArray = channelArray;
                configuration.convertingTime = convertingTime;
                configuration.gainArray = gainArray;
                configuration.polarityArray = polarityArray;
                configuration.channelCount = (byte)channelCount;

                // Start the worker
                this.readWithoutInterruptWorker.RunWorkerAsync(configuration);
            }
        }

        /// <summary>
        /// SE mode is set
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void seRadio_CheckedChanged(object sender, EventArgs e)
        {
            if (this.seRadio.Checked)
            {
                int returnValue = this.board.SetAnalogInputMode(1);
                if (returnValue != 0)
                {
                    MessageBox.Show("SetAnalogInputMode error : " + returnValue.ToString());
                    return;
                }
            }
        }

        /// <summary>
        /// DIFF mode is set
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void diffRadio_CheckedChanged(object sender, EventArgs e)
        {
            if (this.diffRadio.Checked)
            {
                int returnValue = this.board.SetAnalogInputMode(2);
                if (returnValue != 0)
                {
                    MessageBox.Show("SetAnalogInputMode error : " + returnValue.ToString());
                    return;
                }
            }
        }

        /// <summary>
        /// Do the job
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void readWithoutInterruptWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            int returnValue = 0;
            String text = String.Empty;
            Configuration configuration = (Configuration)e.Argument;
            ushort[] values = new ushort[configuration.channelCount];

            // Interrupt
            returnValue = this.board.ReadMoreAnalogInputs(configuration.channelCount,
                configuration.channelArray,
                configuration.gainArray,
                configuration.polarityArray,
                configuration.convertingTime,
                0,      // no interrupt
                0,      // no software trigger
                0,      // no hardware trigger
                values);

            if (returnValue != 0)
            {
                // Error, set the text
                this.Invoke(new SetTextDelegate(this.SetText), "ReadMoreAnalogInputs error : " + returnValue.ToString());
                return;
            }

            // Ok display the values
            for (int i = 0; i < (int)configuration.channelCount; i++)
            {
                text += "Channel " + configuration.channelArray[i].ToString() + " = " + values[i] + "\r\n";
            }
            this.Invoke(new SetTextDelegate(this.SetText), text);
        }
    }
}